<?php

class ACMS_POST_Form_Submit extends ACMS_POST_Form
{
    function post()
    {
        // CSRF対策
        if ( 1
            && config('form_csrf_enable', 'on') !== 'off' 
            && !$this->checkCsrfToken()
        ) {
            $this->Post->set('step', 'repeated');
            return $this->Post;
        }
        if ( isCSRF() ) {
            $this->Post->set('step', 'forbidden');
            return $this->Post;
        }

        // フォーム情報のロード
        $id     = $this->Post->get('id');
        $info   = $this->loadForm($id);
        if ( empty($info) ) {
            userErrorLog('Not Found Form ID.');
            die503('Not Found Form ID.');
        }
        $Form   = $info['data'];
        $Mail   =& $this->mergeMainConfig($Form->getChild('mail'));

        // サーバサイドのバリデーションを実装
        $Option = new Field();
        $Option->overload($Form->getChild('option'));

        // POSTの整形
        $Field  =& $this->extract('field');
        $dup    = $this->buildOptions($Option);
        if( !empty($dup) ) {
            list($fd, $mail) = $dup;
            $Field->setMethod($fd, 'duplication', $this->mailToDouble($info['id'], $mail));
        }

        // バリデート
        $Field->validate(new ACMS_Validator());
        if ( !$this->Post->isValidAll() ) {
            return $this->Post;
        }

        // 連番の更新
        $this->updateCount($info['id'], $Field);
        
        // メール送信
        $log        = $this->sendAutoReply($Mail, $Field);
        $admin_log  = $this->sendToAdministrator($Mail, $Field);

        // ログを記憶
        $this->updateLog($info, $log, $admin_log, $Field);

        return $this->Post;
    }

    /**
     * CSRFトークンのチェック
     * 
     * @return boolean
     */
    function checkCsrfToken()
    {
        $check      = false;
        $Session    = ACMS_Session::singleton();

        if ( $Session->get('formToken') === $this->Post->get('formToken') ) {
            $check  = true;
        }

        $Session->delete('formToken');
        $Session->save();

        return $check;
    }

    /**
     * 連番の更新
     * 
     * @param int $fmid
     * @param Field & $Field
     */
    function updateCount($fmid, & $Field)
    {
        if ( !!$fmid ) {
            $DB = DB::singleton(dsn());
            $SQL = SQL::newUpdate('form');
            $SQL->addWhereOpr('form_id', $fmid);
            $SQL->setUpdate('form_current_serial',
                SQL::newFunction(SQL::newOpr('form_current_serial', 1, '+'), 'LAST_INSERT_ID')
            );
            $Field->set('serialNumber', $DB->query($SQL->get(dsn()), 'seq'));
        } else {
            $Field->set('serialNumber', '');
        }
    }

    /**
     * メール情報のマージ
     * ConfigとPostのメール情報をマージする
     * 
     * @param Field $Config
     * @return Field & $Mail
     */
    function & mergeMainConfig($Config)
    {
        $this->Post->set('acms_field_mail', array(
            'To', 'SubjectTpl', 'BodyTpl', 'BodyHTMLTpl', 'Charset', 'CharsetHTML', 'Obfuscation',
            'Sender', 'From', 'Reply-To', 'Cc', 'Bcc',
            'AdminTo', 'AdminSubjectTpl', 'AdminBodyTpl', 
            'AdminFrom', 'AdminReply-To', 'AdminCc', 'AdminBcc',
        ));
        $Mail   =& $this->extract('acms_field_mail');
        $Mail->overload($Config);

        if ( !!$Mail->get('Obfuscation') ) {
            foreach ( array(
                'To', 'Sender', 'From', 'Reply-To', 'Cc', 'Bcc',
                'AdminTo', 'AdminFrom', 'AdminReply-To', 'AdminCc', 'AdminBcc',
            ) as $field ) {
                $values = array();
                foreach ( $Mail->getArray($field) as $value ) {
                    $values[]   = base64_decode($value);
                }
                $Mail->set($field, $values);
            }
        }

        return $Mail;
    }

    /**
     * 自動返信メール送信
     * 
     * @param Field & $Mail
     * @param Field & $Field
     * @return array
     */
    function sendAutoReply(& $Mail, & $Field)
    {
        $to = $Mail->getArray('To');
        $subjectTpl     = findTemplate($Mail->get('SubjectTpl'));
        $bodyTpl        = findTemplate($Mail->get('BodyTpl'));
        $htmlBodyTpl    = findTemplate($Mail->get('BodyHTMLTpl'));

        $info = $this->getBaseMailInfo($Mail);
        if ( empty($to) || empty($subjectTpl) || empty($bodyTpl) ) {
            return $info;
        }

        $info = array_merge($info, array(
            'from'          => $Mail->get('From'),
            'to'            => $to,
            'subject'       => $this->getMailTxt($subjectTpl, $Field),
            'cc'            => $Mail->getArray('Cc'),
            'bcc'           => $Mail->getArray('Bcc'),
            'reply-to'      => $Mail->getArray('Reply-To'),
            'body'          => $this->getMailTxt($bodyTpl, $Field),
        ));

        $FormMail = new Mail($this->mailConfig());

        // 基本情報を追加
        $this->addBaseMailParam($FormMail, $info);

        // HTMLメール
        if ( $htmlBodyTpl ) {
            $info['body-html'] = $this->getMailTxt($htmlBodyTpl, $Field);
            $this->addHtmlMailParam($FormMail, $info);
        // テキストメール
        } else {
            $this->addTextMailPram($FormMail, $info);
        }
        $FormMail->send();

        return $info;
    }

    /**
     * 管理者宛メール送信
     * 
     * @param Field & $Mail
     * @param Field & $Field
     * @return array
     */
    function sendToAdministrator(& $Mail, & $Field)
    {
        $to = $Mail->getArray('AdminTo');
        $subjectTpl     = findTemplate($Mail->get('AdminSubjectTpl'));
        $bodyTpl        = findTemplate($Mail->get('AdminBodyTpl'));
        $htmlBodyTpl    = findTemplate($Mail->get('AdminBodyHTMLTpl'));
        $attached_files = array();

        $info = $this->getBaseMailInfo($Mail);
        if ( empty($to) || empty($subjectTpl) || empty($bodyTpl) ) {
            return $info;
        }

        $info = array_merge($info, array(
            'from'          => $Mail->get('AdminFrom'),
            'to'            => $to,
            'subject'       => $this->getMailTxt($subjectTpl, $Field),
            'cc'            => $Mail->getArray('AdminCc'),
            'bcc'           => $Mail->getArray('AdminBcc'),
            'reply-to'      => $Mail->getArray('AdminReply-To'),
            'body'          => $this->getMailTxt($bodyTpl, $Field),
            'attached_file' => false,
        ));

        $FormMail = new ACMS_POST_Form_Mail($this->mailConfig());

        // 基本情報を追加
        $this->addBaseMailParam($FormMail, $info);

        // 添付ファイル
        if ( $Mail->get('AdminAttachment') === 'on' ) {
            $attached_files = $this->addAttachedFile($FormMail, $Field);
            if ( count($attached_files) > 0 ) {
                $info['attached_file'] = true;
            }
        }

        // HTMLメール
        if ( $htmlBodyTpl ) {
            $info['body-html'] = $this->getMailTxt($htmlBodyTpl, $Field);
            $this->addHtmlMailParam($FormMail, $info);
        // テキストメール
        } else {
            $this->addTextMailPram($FormMail, $info);
        }
        $FormMail->send();

        // 添付ファイルの削除
        foreach ( $attached_files as $path ) {
            @unlink($path);
            if ( HOOK_ENABLE ) {
                $Hook = ACMS_Hook::singleton();
                $Hook->call('mediaDelete', $path);
            }
        }

        return $info;
    }

    /**
     * メールパラメータのベース
     * 
     * @param Mail $Mail
     * @return array
     */
    function getBaseMailInfo($Mail)
    {
        return array(
            'sender'        => $Mail->get('Sender'),
            'charset'       => $Mail->get('Charset', 'ISO-2022-JP'),
            'charset-html'  => $Mail->get('CharsetHTML', 'UTF-8'),
            'from'          => '',
            'to'            => array(),
            'subject'       => '',
            'cc'            => array(),
            'bcc'           => array(),
            'reply-to'      => array(),
            'body'          => '',
            'body-html'     => '',
            'attached_file' => false,
        );
    }

    /**
     * 基本のメールパラメータを追加
     * 
     * @param Field & $FromMail
     * @param array $info
     */
    function addBaseMailParam(& $FormMail, $info)
    {
        $FormMail->setHeaderReturnPath();
        $FormMail->setHeaderDate();
        $FormMail->setHeaderMessageId();
        $FormMail->setHeader('Sender', $info['sender']);
        $FormMail->setHeader('From', $info['from']);
        $FormMail->setHeader('To', $info['to']);
        $FormMail->setHeader('Subject', $info['subject']);
        $FormMail->setHeader('Cc', $info['cc']);
        $FormMail->setHeader('Bcc', $info['bcc']);
        $FormMail->setHeader('Reply-To', $info['reply-to']);
        $FormMail->setHeader('MIME-Version', '1.0');
    }

    /**
     * テキストメールのパラメータを追加
     * 
     * @param Field & $FromMail
     * @param array $info
     */
    function addTextMailPram(& $FormMail, $info)
    {
        // 添付あり
        if ( $info['attached_file'] ) {
            $RootForm = new Mail();
        // 添付なし
        } else {
            $RootForm =& $FormMail;
        }

        $RootForm->setHeader('Content-Type', 'text/plain', array('charset' => $info['charset']));
        $RootForm->setHeader('Content-Transfer-Encoding', ('ISO-2022-JP' == $info['charset']) ? '7bit' : 'base64');
        $RootForm->setBody($info['body']);

        // 添付あり
        if ( $info['attached_file'] ) {
            $FormMail->addBody($RootForm);
        }
    }

    /**
     * HTMLメールのパラメータを追加
     * 
     * @param Field & $FromMail
     * @param array $info
     */
    function addHtmlMailParam(& $FormMail, $info)
    {
        // 添付あり
        if ( $info['attached_file'] ) {
            $RootForm = new Mail();
        // 添付なし
        } else {
            $RootForm =& $FormMail;
        }

        $RootForm->setHeader('Content-Type', 'multipart/alternative', array(
            'boundary'  => 'ablogcms-'.uniqueString(),
        ));
        $Plain  = new Mail();
        $Plain->setHeader('Content-Type', 'text/plain', array(
            'charset'   => $info['charset'],
        ));
        $Plain->setHeader('Content-Transfer-Encoding', 'base64');
        $Plain->setBody($info['body']);
        $RootForm->addBody($Plain);

        $Html   = new Mail();
        $Html->setHeader('Content-Type', 'text/html', array(
            'charset'   => $info['charset-html'],
        ));
        $Html->setHeader('Content-Transfer-Encoding', 'base64');
        $Html->setBody($info['body-html']);
        $RootForm->addBody($Html);

        // 添付あり
        if ( $info['attached_file'] ) {
            $FormMail->addBody($RootForm);
        }
    }

    /**
     * 添付ファイルのパラメータを追加
     * 
     * @param Field & $FromMail
     * @param Field $Field
     * @return array
     */
    function addAttachedFile(& $FormMail, $Field)
    {
        $attached_files = array();

        foreach ( $Field->listFields() as $fd ) {
            $pathInfo = $this->getAttachedFilePath($Field, $fd);

            if ( $pathInfo === false ) {
                continue;
            }

            $temppath = $pathInfo['temppath'];

            if ( $FormMail->addFile($temppath) !== false ) {
                $attached_files[] = $temppath;
            }
        }

        if ( count($attached_files) > 0 ) {
            $FormMail->setHeader('Content-Type', 'multipart/mixed', array(
                'boundary' => 'ablogcms-'.uniqueString(),
            ));
        }

        return $attached_files;
    }

    /**
     * メールログの保存
     * 
     * @param array $info
     * @param array $log
     * @param Field $Field
     * @param array $admin_log
     */
    function updateLog($info, $log, $admin_log, $Field)
    {
        if ( empty($info) ) {
            return false;
        }

        $fmid   = $info['id'];
        $fmbid  = $info['bid'];
        $fmeid  = $this->Post->get('eid');

        $DB     = DB::singleton(dsn());

        $SQL    = SQL::newInsert('log_form');
        $SQL->addInsert('log_form_datetime', date('Y-m-d H:i:s', REQUEST_TIME));
        $SQL->addInsert('log_form_remote_addr', REMOTE_ADDR);
        $SQL->addInsert('log_form_user_agent', UA);
        $SQL->addInsert('log_form_mail_to', strval(join(', ', $log['to'])));
        $SQL->addInsert('log_form_mail_subject', acmsSerialize(strval($log['subject'])));
        $SQL->addInsert('log_form_mail_subject_admin', acmsSerialize(strval($admin_log['subject'])));
        $SQL->addInsert('log_form_mail_body', acmsSerialize(strval($log['body'])));
        $SQL->addInsert('log_form_mail_body_admin', acmsSerialize(strval($admin_log['body'])));
        $SQL->addInsert('log_form_data', acmsSerialize($Field));
        $SQL->addInsert('log_form_serial', $Field->get('serialNumber'));
        $SQL->addInsert('log_form_version', 1);

        $SQL->addInsert('log_form_entry_id', $fmeid);
        $SQL->addInsert('log_form_form_id', $fmid);
        $SQL->addInsert('log_form_blog_id', $fmbid);
        $DB->query($SQL->get(dsn()), 'exec');

        if ( HOOK_ENABLE ) {
            $Hook = ACMS_Hook::singleton();
            $Hook->call('formSubmit', array($log, $admin_log));
        }
    }
}
